// Description: a node that generates the distance to a node
//
//<b>notes:</b>
//<ul>
//<li> derived nodes inherit all messages from their base classes
//<li> see reset method for default settings
//<li> see the <a href="interpolate">interpolate</a> node for more information
//</ul>
//
// Category: Math
// Author: Alex Hill
// Revision: 11/01/01
//
#include <Performer/pf/pfGroup.h>
#include <ygWorld.h>
#include <ygUser.h>
#include <ygNodeDB.h>
#include <ygNetMessages.h>
#include "distance.h"

#define USER_NONE 100
#define USER_HEAD 101
#define USER_BODY 102
#define USER_WAND 103

extern "C" ygNode* APIENTRY construct_distance(const char* name,bool master) { return new distance_(name,master); }
char* distance_dependencies = "YG DSO Dependencies: interpolate";

distance_::distance_(const char* name,bool master) : interpolate(name,master)
    {
    setClassName("distance");
    setPfNode(new pfGroup);
    node = NULL;
    mode = USER_NONE;
    wandNum = 0;
    bounding = false;
    addNetMessage("node",&nodeName,YG_NET_NODE);
    addNetMessage("node",&mode,YG_NET_INT);
    addNetMessageMap("node","none",USER_NONE);
    addNetMessageMap("node","head",USER_HEAD);
    addNetMessageMap("node","body",USER_BODY);
    addNetMessageMap("node","wand",USER_WAND);
    addNetMessage("node",&wandNum,YG_NET_INT);
    addNetMessage("bounding",&bounding,YG_NET_BOOL_NEGATE);
    }

distance_::~distance_(void)
    {
    }

void distance_::reset(void)
    {
    //clear node name
    node = NULL;
    nodeName.clear();
    mode = USER_NONE;
    wandNum = 0;
    bounding = false;
    interpolate::reset();
    }

void distance_::message(const ygMessage& msg)
    {
    //clear or set the node and optional user body part
    if (msg == "node")
        {
        //if number of arguments is greater than zero
        if (nodeName.length() > 0)
            {
            node = ygNodeDB::find(nodeName);
            //if node type is user then
            if (node && node->isOfClass("ygUser"))
                {
                ygUser* user = (ygUser*)node;
                //if second argument is "head"
                if (mode == USER_HEAD)
                    {
                    //set node to user head
                    if (user->head())
                        node = user->head();
                    }
                //if second argument is "body"
                else if (mode == USER_BODY)
                    {
                    //for each user child
                    for (int i=0; i<user->numChildren(); i++)
                        {
                        ygNode* currNode = user->child(i);
                        //if child is of type body then set node
                        if (currNode->isOfClass("body"))
                            node = currNode;
                        }
                    }
                //if second argument is "wand"
                else if (mode == USER_WAND)
                    {
                    //if user has a wand then
                    if (user->wand(wandNum))
                        {
                        //use specified wand number
                        node = user->wand(wandNum);
                        }
                    }
                else
                    node = user;
                }
            }
        //else, clear the node name
        else
            node = NULL;
        }
    //set the bounding sphere reporting mode
    else if (msg == "bounding")
        {
        return;
        }
    else
        interpolate::message(msg);
    }

void distance_::app(void)
    {
    //if node is set
    if (node)
        {
        //calculate user distance
        pfVec3 pos,thisPos,vec;
        pos = node->origin();
       	thisPos = origin();
        vec = pos - thisPos;
        float distance_;
        distance_ = vec.length();
        if (bounding)
            {
            pfSphere sphere;
            node->pfnode()->getBound(&sphere);
            distance_ = distance_ - sphere.radius;
            }
        //calculate interpolated array values
        for (int i=0;i<size;i++)
            values[i] = startValues[i] + distance_*(endValues[i]-startValues[i]);
        }
    interpolate::app();
    }
