// Description: a node that switches intersection and draw state
//
//<b>notes:</b>
//<ul>
//<li> all parent node messages are inherited
//<li> see reset method for default settings
//</ul>
//
// Category: Attributes
// Author: Alex Hill
// Revision: 03/01/05 Alex Hill
//
#include <Performer/pf/pfGroup.h>
#include <Performer/pfdu.h>
#include <ygNetMessages.h>
#include <ygWorld.h>
#include "geometrySwitch.h"

extern "C" ygNode* APIENTRY construct_geometrySwitch(const char* name,bool master) { return new geometrySwitch(name,master); }

geometrySwitch::geometrySwitch(const char* name,bool master) : ygNode(name,master)
    {
    setClassName("geometrySwitch",true);
    draw = DRAW_ON;
    isect = YG_ISECT_ALLMASK;
    addNetMessage("draw",&draw,YG_NET_INT);
    addNetMessageMap("draw","false",DRAW_OFF);
    addNetMessageMap("draw","true",DRAW_ON);
    addNetMessageMap("draw","local",DRAW_LOCAL);
    addNetMessageMap("draw","remote",DRAW_REMOTE);
    addNetMessage("intersect",&isect,YG_NET_INT);
    addNetMessageMap("intersect","all",YG_ISECT_ALLMASK);
    addNetMessageMap("intersect","wall",YG_ISECT_WALLMASK);
    addNetMessageMap("intersect","floor",YG_ISECT_FLOORMASK);
    addNetMessageMap("intersect","grab",YG_ISECT_GRABMASK);
    addNetMessageMap("intersect","debug",YG_ISECT_DEBUGMASK);
    addNetMessageMap("intersect","off",0);
    }

geometrySwitch::~geometrySwitch(void)
    {
    }


void geometrySwitch::reset(void)
    {
    //set the draw flag to true
    if (draw != DRAW_ON)
        {
        resetNetMessage("draw");
        setDraw(draw);
        }
    //set the intersect flag to "all"
    if (isect != YG_ISECT_ALLMASK)
        {
        resetNetMessage("intersect");
        setIntersect(isect);
        }
    ygNode::reset();
    }


void geometrySwitch::message(const ygMessage& msg)
    {
    //set the object draw state
    if (msg == "draw")
        {
        setDraw(draw);
        }
    //set the object intersect state
    else if (msg == "intersect")
        {
        setIntersect(isect);
        }
    else
        ygNode::message(msg);
    }


void geometrySwitch::setDraw(int val)
    {
    draw = val;
    if (val == DRAW_ON || (isNetMaster() && val == DRAW_LOCAL) || (!isNetMaster() && val == DRAW_REMOTE))
        pfnode()->setTravMask(PFTRAV_DRAW,0xffffffff,PFTRAV_SELF,PF_SET);
    else
        pfnode()->setTravMask(PFTRAV_DRAW,0,PFTRAV_SELF,PF_SET);
    }


void geometrySwitch::setIntersect(int val)
    {
    isect = val;
    pfnode()->setTravMask(PFTRAV_ISECT,(uint)isect,PFTRAV_SELF,PF_SET);
    }



