// Description: creates events from keyboard input
//
//<b>notes:</b>
//<ul>
//<li> derived nodes inherit all messages from their base classes
//<li> see reset method for default settings
//<li> the keys detected include most keyboard keys except function and keypad keys
//</ul>
//
// Category: Trigger
// Author: Dave Pape
//           11/01/01
//           09/01/05 Alex Hill - added majority of keyboard keys and added "key" event
// Revision: 07/01/06 Alex Hill - added function keys
//
#include <pfcave.h>
#include "keyboard.h"

extern "C" ygNode* APIENTRY construct_keyboard(const char* name,bool master) { return new keyboard(name,master); }

#define NUMKEYS 75

static struct { CAVEDevice dev; char *label; } keyboardKeys[NUMKEYS] =
    {
        { CAVE_AKEY, "A" },
        { CAVE_BKEY, "B" },
        { CAVE_CKEY, "C" },
        { CAVE_DKEY, "D" },
        { CAVE_EKEY, "E" },
        { CAVE_FKEY, "F" },
        { CAVE_GKEY, "G" },
        { CAVE_HKEY, "H" },
        { CAVE_IKEY, "I" },
        { CAVE_JKEY, "J" },
        { CAVE_KKEY, "K" },
        { CAVE_LKEY, "L" },
        { CAVE_MKEY, "M" },
        { CAVE_NKEY, "N" },
        { CAVE_OKEY, "O" },
        { CAVE_PKEY, "P" },
        { CAVE_QKEY, "Q" },
        { CAVE_RKEY, "R" },
        { CAVE_SKEY, "S" },
        { CAVE_TKEY, "T" },
        { CAVE_UKEY, "U" },
        { CAVE_VKEY, "V" },
        { CAVE_WKEY, "W" },
        { CAVE_XKEY, "X" },
        { CAVE_YKEY, "Y" },
        { CAVE_ZKEY, "Z" },
        { CAVE_ZEROKEY, "0" },
        { CAVE_ONEKEY, "1" },
        { CAVE_TWOKEY, "2" },
        { CAVE_THREEKEY, "3" },
        { CAVE_FOURKEY, "4" },
        { CAVE_FIVEKEY, "5" },
        { CAVE_SIXKEY, "6" },
        { CAVE_SEVENKEY, "7" },
        { CAVE_EIGHTKEY, "8" },
        { CAVE_NINEKEY, "9" },
        { CAVE_BREAKKEY, "break" },
        { CAVE_CTRLKEY, "rightcontrol" },
        { CAVE_LEFTCTRLKEY, "leftcontrol" },
        { CAVE_CAPSLOCKKEY, "capslock" },
        { CAVE_RIGHTSHIFTKEY, "rightshift" },
        { CAVE_LEFTSHIFTKEY, "leftshift" },
        { CAVE_ESCKEY, "escape" },
        { CAVE_TABKEY, "tab" },
        { CAVE_RETKEY, "return" },
        { CAVE_SPACEKEY, "space" },
        { CAVE_BACKSPACEKEY, "backspace" },
        { CAVE_DELKEY, "delete" },
        { CAVE_SEMICOLONKEY, "semicolon" },
        { CAVE_PERIODKEY, "period" },
        { CAVE_COMMAKEY, "comma" },
        { CAVE_QUOTEKEY, "apostrophe" },
        { CAVE_ACCENTGRAVEKEY, "accent" },
        { CAVE_MINUSKEY, "minus" },
        { CAVE_VIRGULEKEY, "forwardslash" },
        { CAVE_BACKSLASHKEY, "backslash" },
        { CAVE_EQUALKEY, "equal" },
        { CAVE_LEFTBRACKETKEY, "leftbracket" },
        { CAVE_RIGHTBRACKETKEY, "rightbracket" },
        { CAVE_LEFTARROWKEY, "leftarrow" },
        { CAVE_DOWNARROWKEY, "downarrow" },
        { CAVE_RIGHTARROWKEY, "rightarrow" },
        { CAVE_UPARROWKEY, "uparrow" },
        { CAVE_F1KEY, "f1" },
        { CAVE_F2KEY, "f2" },
        { CAVE_F3KEY, "f3" },
        { CAVE_F4KEY, "f4" },
        { CAVE_F5KEY, "f5" },
        { CAVE_F6KEY, "f6" },
        { CAVE_F7KEY, "f7" },
        { CAVE_F8KEY, "f8" },
        { CAVE_F9KEY, "f9" },
        { CAVE_F10KEY, "f10" },
        { CAVE_F11KEY, "f11" },
        { CAVE_F12KEY, "f12" }
    };

struct _keyboardPrivateData
    {
    int prevState[NUMKEYS];
    };


keyboard::keyboard(const char* name,bool master) : ygNode(name,master)
    {
    setClassName("keyboard");
    p_ = new struct _keyboardPrivateData;
    addEvent("key");
    }

keyboard::~keyboard(void)
    {
    delete p_;
    }


void keyboard::reset(void)
    {
    int i;
    //set the previous state of all keys to 0
    for (i=0; i < NUMKEYS; i++)
        p_->prevState[i] = 0;
    ygNode::reset();
    }


void keyboard::app(void)
    {
    int i;
    //for each possible key
    for (i=0; i < NUMKEYS; i++)
        {
        int state = CAVEgetbutton(keyboardKeys[i].dev);
        //if the state has changed then generate a key event
        if (state && (!p_->prevState[i]))
            {
            ygString event(keyboardKeys[i].label);
            event += "key";
            //the key concatenated with the word "key" (i.e. akey, 1key)
            eventOccurred(event);
            ygString args("name=");
            args += keyboardKeys[i].label;
            //the key name has been pressed
            eventOccurred("key",args);
            }
        p_->prevState[i] = state;
        }
    ygNode::app();
    }
