// Description: a value node that performs common mathematic operations
//
//<b>notes:</b>
//<ul>
//<li> derived nodes inherit all messages from their base classes
//<li> see reset method for default settings
//<li> see the <a href="value.html">value</a> node for more information
//</ul>
//
// Category: Math
// Author: Alex Hill
// 11/01/02
// Revision: 04/01/06 Alex Hill - now reads child node and converts to given function
//
#include <ygNetMessages.h>
#include "mathematics.h"

#ifdef WIN32
#include <stdlib.h>
//#include <time.h>
#endif

#define FUNC_NONE 0
#define FUNC_ABS 1
#define FUNC_SIN 2
#define FUNC_COS 3
#define FUNC_SQRT 4

#ifdef WIN32
#define drand48()   ((double) ((double) rand() / (double) RAND_MAX))
#endif

extern "C" ygNode* APIENTRY construct_mathematics(const char* name,bool master) { return new mathematics(name,master); }
char* mathematics_dependencies = "YG DSO Dependencies: interpolate";

mathematics::mathematics(const char* name,bool master) : interpolate(name,master)
    {
    setClassName("mathematics");
#ifdef WIN32    
    srand(time(NULL));
#endif    
    function_ = FUNC_NONE;
    addNetMessage("function",&function_,YG_NET_INT);
    addNetMessageMap("function","none",FUNC_NONE);
    addNetMessageMap("function","abs",FUNC_ABS);
    addNetMessageMap("function","sin",FUNC_SIN);
    addNetMessageMap("function","cos",FUNC_COS);
    addNetMessageMap("function","sqrt",FUNC_SQRT);
    }

void mathematics::message(const ygMessage& msg)
    {
    //set a single array element to the absolute value of the input
    if (msg == "abs")
        {
        if (msg.numArgs() > 0)
            {
            float newValue = msg.floatArg(0);
            if (msg.numArgs() > 1) 
                {
                int element = msg.intArg(1);
                if (element >= 0)
                    {    
                    if (element >= size)
                        setValueSize(element+1);
                    setInterpolate(fabs(newValue),element);
                    }
                else
                    msg.error(name(),"(index out of range)");
                }
            else
                {
                for (int i=0;i<size;i++)
                    setInterpolate(fabs(newValue),i);
                }
            }
        function_ = FUNC_ABS;
        }
    //set a single array element to the sine of the input
    else if (msg == "sin")
        {
        if (msg.numArgs() > 0)
            {
            float newValue = msg.floatArg(0);
            if (msg.numArgs() > 1) 
                {
                int element = msg.intArg(1);
                if (element >= 0)
                    {    
                    if (element >= size)
                        setValueSize(element+1);
                    setInterpolate(sinf(newValue),element);
                    }
                else
                    msg.error(name(),"(index out of range)");
                }
            else
                {
                for (int i=0;i<size;i++)
                    setInterpolate(sinf(newValue),i);
                }
            }
        function_ = FUNC_SIN;
        }
    //set a single array element to the cosine of the input
    else if (msg == "cos")
        {
        if (msg.numArgs() > 0)
            {
            float newValue = msg.floatArg(0);
            if (msg.numArgs() > 1) 
                {
                int element = msg.intArg(1);
                if (element >= 0)
                    {    
                    if (element >= size)
                        setValueSize(element+1);
                    setInterpolate(cosf(newValue),element);
                    }
                else
                    msg.error(name(),"(index out of range)");
                }
            else
                {
                for (int i=0;i<size;i++)
                    setInterpolate(cosf(newValue),i);
                }
            }
        function_ = FUNC_COS;
        }
    //set a single array element to a random number
    else if (msg == "rand")
        {
        if (msg.numArgs() < 2)
            {
            if (msg.numArgs() == 1) 
                {
                int element = msg.intArg(0);
                if (element >= 0)
                    {    
                    if (element >= size)
                        setValueSize(element+1);
                    setInterpolate(drand48(),element);
                    }
                else
                    msg.error(name(),"(index out of range)");
                }
            else
                {
                for (int i=0;i<size;i++)
                    setInterpolate(drand48(),i);
                }
            }
        else
            msg.error(name(),"(wrong number of arguments)");
        }
    //set a single array element to the square root of the positive input
    else if (msg == "sqrt")
        {
        if (msg.numArgs() > 0)
            {
            float newValue = msg.floatArg(0);
            if (newValue >= 0.0)
                {
                if (msg.numArgs() > 1) 
                    {
                    int element = msg.intArg(1);
                    if (element >= 0)
                        {    
                        if (element >= size)
                            setValueSize(element+1);
                        setInterpolate(sqrt(newValue),element);
                        }
                    else
                        msg.error(name(),"(index out of range)");
                    }
                else
                    {
                    for (int i=0;i<size;i++)
                        setInterpolate(sqrt(newValue),i);
                    }
                }
            }
        function_ = FUNC_SQRT;
        }
    else if (msg == "function")
        {
        return;
        }
    else
        interpolate::message(msg);
    }

void mathematics::app(void)
    {
    //if this node has size greater than zero and has children
    if (size > 0 && numChildren() > 0 && child(numChildren()-1) && child(numChildren()-1)->isOfClass("value"))
        {
        value* currChild = (value*)child(numChildren()-1);
        int currSize = currChild->getSize();
        //if child size is greater than one
        if (currSize > 1)
            {
            //for each value in the last child node do
            for (int i=0;i<currSize;i++)
                {
                //if the element is less than the size of this node then
                if (i < size)
                    {
                    //set the element value to the interpolated value of the child value
                    if (function_ == FUNC_ABS)
                        setInterpolate(fabs(currChild->getValue(i)),i);
                    else if (function_ == FUNC_SIN)
                        setInterpolate(sinf(currChild->getValue(i)),i);
                    else if (function_ == FUNC_COS)
                        setInterpolate(cosf(currChild->getValue(i)),i);
                    else if (function_ == FUNC_SQRT)
                        setInterpolate(sqrt(currChild->getValue(i)),i);
                    }
                }
            }
        else
            {
            //retrieve the value of the child node
            float inputValue = currChild->getValue();
            //for each element in the array do
            for (int i=0;i<size;i++)
                {
                //set the element value to the interpolated value of the value
                if (function_ == FUNC_ABS)
                    setInterpolate(fabs(inputValue),i);
                else if (function_ == FUNC_SIN)
                    setInterpolate(sinf(inputValue),i);
                else if (function_ == FUNC_COS)
                    setInterpolate(cosf(inputValue),i);
                else if (function_ == FUNC_SQRT)
                    setInterpolate(sqrt(inputValue),i);
                }
            }
            
        }
    value::app();
    }
