// Description: generates events describing node messages
//
//<b>notes:</b>
//<ul>
//<li> derived nodes inherit all messages from their base classes
//<li> see reset method for default settings
//</ul>
//
// Category: System
// Author: Alex Hill
// Revision: 09/15/04 Alex Hill
//
#include <ygNodeDB.h>
#include <ygNetMessages.h>
#include "nodeMessages.h"

using namespace std;

extern "C" ygNode* APIENTRY construct_nodeMessages(const char* name,bool master) { return new nodeMessages(name,master); }

nodeMessages::nodeMessages(const char* name,bool master) : ygNode(name,master)
    {
    setClassName("nodeMessages");
    node = NULL;
    manipulate = true;
    grab = true;
    addNetMessage("node",&nodeName,YG_NET_STRING);
    addNetMessage("messages",&messageName,YG_NET_STRING);
    addNetMessage("manipulate",&manipulate,YG_NET_BOOL);
    addNetMessage("grab",&grab,YG_NET_BOOL);
    manipulateMap["ygTransform"].resize(3);
    manipulateMap["ygTransform"][0].push_back("position");
    manipulateMap["ygTransform"][0].push_back("auto");
    manipulateMap["ygTransform"][0].push_back("position");
    manipulateMap["ygTransform"][0].push_back("position0");
    manipulateMap["ygTransform"][0].push_back("position1");
    manipulateMap["ygTransform"][0].push_back("position2");
    manipulateMap["ygTransform"][1].push_back("orientation");
    manipulateMap["ygTransform"][1].push_back("auto");
    manipulateMap["ygTransform"][1].push_back("orientation");
    manipulateMap["ygTransform"][1].push_back("orientation0");
    manipulateMap["ygTransform"][1].push_back("orientation1");
    manipulateMap["ygTransform"][1].push_back("orientation2");
    manipulateMap["ygTransform"][2].push_back("size");
    manipulateMap["ygTransform"][2].push_back("auto");
    manipulateMap["ygTransform"][2].push_back("sizeall");
    manipulateMap["ygTransform"][2].push_back("size0");
    manipulateMap["ygTransform"][2].push_back("size1");
    manipulateMap["ygTransform"][2].push_back("size2");
    addEvent("node");
    addEvent("messages");
    addEvent("events");
    addEvent("filter");
    addEvent("manipulate");
    addEvent("grab");
    addEvent("manipulatable");
    addEvent("map");
    }
    
void nodeMessages::reset(void)
    {
    node = NULL;
    lastNode.clear();
    lastMessage.clear();
    resetNetMessage("node");
    resetNetMessage("messages");
    resetNetMessage("manipulate");
    resetNetMessage("grab");
    ygNode::reset();
    }

void nodeMessages::message(const ygMessage& msg)
    {
    //generate events with message names
    if (msg == "node")
        {
        checkNode();
        }
    //generate events with message arguments
    else if (msg == "messages")
        {
        checkMessages();
        }
    //generate events with message names
    else if (msg == "manipulate")
        {
        checkManipulate();
        }
    //set the grab flag
    else if (msg == "grab")
        {
        return;
        }
    //generate events with message names
    else if (msg == "map")
        {
        checkMap();
        }
    //generate events with event names
    else if (msg == "events")
        {
        checkEvents();
        }
    else
        ygNode::message(msg);
    }

void nodeMessages::checkNode(void)
    {
    if (nodeName.length() > 0 && nodeName != lastNode)
        {
        node = ygNodeDB::find(nodeName);
        lastMessage.clear();
        if (node)
            {
            int found=0;
            if (manipulate)
                {
                map<ygString,vector<vector<ygString> > >::const_iterator i;
                for (i=manipulateMap.begin(); i != manipulateMap.end(); ++i)
                    {
                    if ((*i).first == node->className())
                        {
                        found = 1;
                        eventOccurred("manipulatable");
                        }
                    }
                }
            char val[32];
            ygString args;
            const vector<ygMessage*> messages = node->getNetMessages();
            int i=0;
            for (int j=0;j < messages.size(); j++)
                {
                if (messages[j]->message() != "edit")
                    {
                    sprintf(val,"%d",i+found);
                    args = "number=";
                    args += val;
                    args += " name=";
                    args += messages[j]->message();
                    eventOccurred("node",args);
                    i++;
                    }
                }
            sprintf(val,"%d",i+found);
            args = "number=";
            args += val;
            args += " name=create";
            eventOccurred("node",args);
            i++;
            if (node->className() != "ygWorld")
                {
                sprintf(val,"%d",i+found);
                args = "number=";
                args += val;
                args += " name=addChild";
                eventOccurred("node",args);
                i++;
                sprintf(val,"%d",i+found);
                args = "number=";
                args += val;
                args += " name=edit";
                eventOccurred("node",args);
                i++;
                }
            sprintf(val,"%d",i+found);
            args = "number=";
            args += val;
            args += " name=event";
            eventOccurred("node",args);
            i++;
            sprintf(val,"%d",i+found);
            args = "number=";
            args += val;
            args += " name=when";
            eventOccurred("node",args);
            i++;
            sprintf(val,"%d",i+found);
            args = "number=";
            args += val;
            args += " name=reset";
            eventOccurred("node",args);
            }
        }    
    lastNode = nodeName;
    }

void nodeMessages::checkMessages(void)
    {
    if (messageName.length() > 0 && node && messageName != lastMessage)
        {
         if (messageName == "create")
            {
            ygString args;
            args = "number=0";
            args += " type=";
            char val[32];
            sprintf(val,"%d",YG_NET_CLASS);
            args += val;
            args += " data=";
            eventOccurred("messages",args);
            }
         else if (messageName == "addChild")
            {
            ygString args;
            args = "number=0";
            args += " type=";
            char val[32];
            sprintf(val,"%d",YG_NET_NODE);
            args += val;
            args += " data=";
            eventOccurred("messages",args);
            }
         else if (messageName == "edit")
            {
            ygString args;
            args = "number=0";
            args += " type=";
            char val[32];
            sprintf(val,"%d",YG_NET_BOOL_NEGATE);
            args += val;
            args += " data=";
            if (node->debugDisplay())
                args += "true";
            else
                args += "false";
            eventOccurred("messages",args);
            }
         else if (messageName == "when")
            {
            const vector<ygString>& events = node->getEvents();
            ygString args;
            int i;
            for (i=0; i < events.size(); i++)
                {
                args = "number=";
                char val[32];
                sprintf(val,"%d",i);
                args += val;
                args += " type=";
                sprintf(val,"%d",YG_NET_WHEN);
                args += val;
                args += " data=";
                args += events[i];
                eventOccurred("messages",args);
                }
            args = "number=";
            char val[32];
            sprintf(val,"%d",i);
            args += val;
            args += " type=";
            sprintf(val,"%d",YG_NET_WHEN);
            args += val;
            args += " data=";
            args += node->getNextEvent();
            eventOccurred("messages",args);
            }
        else if (messageName == "event")
            {
            ygString args;
            args = "number=0";
            args += " type=";
            char val[32];
            sprintf(val,"%d",YG_NET_EVENT);
            args += val;
            args += " data=";
            eventOccurred("messages",args);
            }
        else if (messageName == "reset")
            {
            ygString args;
            args = "number=0";
            args += " type=";
            char val[32];
            sprintf(val,"%d",YG_NET_BOOL_NEGATE);
            args += val;
            args += " data=false";
            eventOccurred("messages",args);
            }
        else
            {
            const vector<ygMessage*> messages = node->getNetMessages();
            for (int i=0;i < messages.size(); i++)
                {
                if (messages[i]->message() == messageName)
                    {
                    int argNum=0;
                    for (int j=0; j < messages[i]->numArgTypes(); j++)
                        {
                        ygString args;
                        args = "number=";
                        char val[32];
                        sprintf(val,"%d",j);
                        args += val;
                        args += " type=";
                        int type = messages[i]->typeArg(j);
                        sprintf(val,"%d",type);
                        args += val;
                        args += " data=\"";
                        if (type >= YG_NET_CLASS && type <= YG_NET_STRING_DELETE)
                            {
                            args += messages[i]->stringArg(argNum++);
                            if (j == messages[i]->numArgTypes()-1)
                                {
                                int size = messages[i]->numArgs();
                                for (int k=argNum; k < size; k++)
                                    {
                                    args += " ";
                                    args += messages[i]->stringArg(k);
                                    }
                                argNum = size;
                                }
                            }
                        else if (type == YG_NET_INT || type == YG_NET_FLOAT || type == YG_NET_BOOL || type == YG_NET_BOOL_NEGATE)
                            {
                            args += messages[i]->stringArg(argNum++);
                            }
                        else if (type == YG_NET_VEC2)
                            {
                            args += messages[i]->stringArg(argNum++);
                            args += " ";
                            args += messages[i]->stringArg(argNum++);
                            }
                        else if (type == YG_NET_VEC3)
                            {
                            args += messages[i]->stringArg(argNum++);
                            args += " ";
                            args += messages[i]->stringArg(argNum++);
                            args += " ";
                            args += messages[i]->stringArg(argNum++);
                            }
                        else if (type == YG_NET_VEC4)
                            {
                            args += messages[i]->stringArg(argNum++);
                            args += " ";
                            args += messages[i]->stringArg(argNum++);
                            args += " ";
                            args += messages[i]->stringArg(argNum++);
                            args += " ";
                            args += messages[i]->stringArg(argNum++);
                            }
                        else if (type == YG_NET_VECTOR)
                            {
                            if (j == messages[i]->numArgTypes()-1)
                                {
                                int size = messages[i]->numArgs();
                                for (int k=argNum; k < size; k++)
                                    {
                                    if (k != argNum)
                                        args += " ";
                                    args += messages[i]->stringArg(k);
                                    }
                                argNum = size;
                                }
                            }
                        else if (type == YG_NET_MATRIX)
                            {
                            int size = argNum + 16;
                            for (int k=argNum; k < messages[i]->numArgs(); k++)
                                {
                                args += messages[i]->stringArg(k);
                                if (k != size-1)
                                    args += " ";
                                }
                            argNum += 16;
                            }
                        else if (type == YG_NET_COORD)
                            {
                            int size = argNum + 6;
                            for (int k=argNum; k < size; k++)
                                {
                                args += messages[i]->stringArg(k);
                                if (k != size-1)
                                    args += " ";
                                }
                            argNum += 6;
                            }
                        args += "\"";
                        eventOccurred("messages",args);
                        }
                    }
                }
            }
        }
    lastMessage = messageName;
    }

void nodeMessages::checkManipulate(void)
    {
    if (manipulate && node)
        {
        int j;
        map<ygString,vector<vector<ygString> > >::const_iterator i;
        for (i=manipulateMap.begin(); i != manipulateMap.end(); ++i)
            {
            const vector<vector<ygString> >& message = (*i).second;
            if ((*i).first == node->className())
                {
                for (j=0;j<message.size();j++)
                    {
                    ygString args("number=");
                    char val[32];
                    sprintf(val,"%d",j);
                    args += val;
                    const vector<ygString>& arguments = message[j];
                    args += " name=";
                    args += arguments[0];
                    args += " axis=";
                    args += arguments[1];
                    args += " mode=";
                    args += arguments[2];
                    for (int k=3;k<arguments.size();k++)
                        {
                        args += " argument";
                        sprintf(val,"%d",k-3);
                        args += val;
                        args += "=";
                        args += arguments[k];
                        }
                    if (node->isOfClass("ygTransform"))
                        eventOccurred("filter",args);
                    else
                        eventOccurred("manipulate",args);
                    }
                }
            }
        if (grab && node->isOfClass("ygTransform"))
            {
            ygString args("number=");
            char val[32];
            sprintf(val,"%d",j);
            args += val;
            eventOccurred("grab",args);
            }
        }
    }


void nodeMessages::checkMap(void)
    {
    if (node)
        {
        const vector<ygString> mapping = node->getNetMessageMap(messageName);
        for (int i=0;i < mapping.size(); i++)
            {
            ygString args;
            args = "number=";
            char val[32];
            sprintf(val,"%d",i);
            args += val;
            args += " name=";
            args += mapping[i];
            eventOccurred("map",args);
            }
        }
    }

void nodeMessages::checkEvents(void)
    {
    if (node)
        {
        const vector<ygString>& events = node->getEvents();
        for (int i=0; i < events.size(); i++)
            {
            ygString args;
            args = "number=";
            char val[32];
            sprintf(val,"%d",i);
            args += val;
            args += " name=";
            args += events[i];
            eventOccurred("events",args);
            }
        }
    }
