// Description: a space that generates position events in the presence of a node type
//
//<b>notes:</b>
//<ul>
//<li> derived nodes inherit all messages from their base classes
//<li> see reset method for default settings
//<li> this node generates events with the name of the node detected
//<li> the position mode determines if the node position is local to the object or relative to world coordinates
//</ul>
//
// Category: Trigger
// Author: Dave Pape
//           11/01/01
// Revision: 09/01/04 Alex Hill - changed to detect by actual class name instead of alias used to create the node
//
#include <ygNodeDB.h>
#include <ygUtil.h>
#include <ygWorld.h>
#include <ygNetMessages.h>
#include "nodeTrigger.h"

using namespace std;

extern "C" ygNode* APIENTRY construct_nodeTrigger(const char* name,bool master) { return new nodeTrigger(name,master); }


nodeTrigger::nodeTrigger(const char* name,bool master) : ygSpace(name,master)
    {
    setClassName("nodeTrigger");
    debugFlag("inside",&debugInside);
    local = true;
    addNetMessage("local",&local,YG_NET_BOOL);
    addNetMessage("detect",&detectType,YG_NET_CLASS);
    addEvent("enter");
    addEvent("exit");
    addEvent("inside");
    }

nodeTrigger::~nodeTrigger(void)
    {
    }


void nodeTrigger::reset(void)
    {
    //clear set of nodes previously inside
    prevInside.clear();
    //set reporting mode to local
    local = true;
    lastFrame = ygWorld::frameNumber();
    ygSpace::reset();
    }


void nodeTrigger::message(const ygMessage& msg)
    {
    //set the type of node to detect
    if (msg == "detect")
        {
        return;
        }
    //set position reporting mode
    else if (msg == "local")
        {
        return;
        }
    else
        ygSpace::message(msg);
    }


void nodeTrigger::setDetectType(const ygString& s)
    {
    detectType = s;
    }


void nodeTrigger::app(void)
    {
    set<ygNode*> inside;
    int i;
    if (lastFrame < (ygWorld::frameNumber()-1))
        prevInside.clear();
    lastFrame = ygWorld::frameNumber();
    const vector<ygNode*>& nodes = ygNodeDB::nodesOfType(detectType);
    //for each node of detect type in the world
    for (i=0; i < nodes.size(); i++)
        {
        //if node is within the space
        pfVec3 pos = nodes[i]->origin(this);
        if (contains(pos))
            {
            if (debugInside)
                cout << name() << ": " << nodes[i]->name() << " is inside\n";
            //add to set of nodes inside
            inside.insert(inside.begin(),nodes[i]);
            ygString args("node=");
            args += nodes[i]->name();
            //if inside but not previously then generate enter event
            if (prevInside.find(nodes[i]) == prevInside.end())
                {
                //the node has entered the space
                eventOccurred("enter",args);
                }
            if (!local)
                pos = nodes[i]->origin();
            char value[32];
            args += " xpos=";
            sprintf(value,"%f",pos[0]);
            args += value;
            args += " ypos=";
            sprintf(value,"%f",pos[1]);
            args += value;
            args += " zpos=";
            sprintf(value,"%f",pos[2]);
            args += value;
            //a node is inside the space
            eventOccurred("inside",args);
            }
        }
    set<ygNode*>::const_iterator iter;
    //for each node previously inside
    for (iter = prevInside.begin(); iter != prevInside.end();
         ++iter)
        //if not currently inside then generate exit event
        if (inside.find(*iter) == inside.end())
            {
            ygString args("node=");
            args += (*iter)->name();
            //the node has exited the space
            eventOccurred("exit",args);
            }
    prevInside = inside;
    ygSpace::app();
    }
