// Description: creates another instance of Performer geometry
//
//<b>notes:</b>
//<ul>
//<li> derived nodes inherit all messages from their base classes
//<li> see reset method for default settings
//<li> all geometry generated below the given node will drawn
//<li> this node can be used to duplicate complicated geometry without duplicating the nodes that generate it
//</ul>
//
// Category: Geometry
// Author: Alex Hill
//           11/01/01
// Revision: 07/20/05 Alex Hill - added functionality to reference a ygTransform node
//
#include <Performer/pf/pfGroup.h>
#include <Performer/pf/pfDCS.h>
#include <ygNetMessages.h>
#include <ygNodeDB.h>
#include <ygWorld.h>
#include "reference.h"

extern "C" ygNode* APIENTRY construct_reference(const char* name,bool master) { return new reference(name,master); }


reference::reference(const char* name,bool master) : ygNode(name,master,false)
    {
    setClassName("reference",true);
    //create new pfDCS and set as pfnode
    dcs = new pfDCS;
    setPfNode(dcs);
    //distribute the nodeName string
    addNetMessage("node",&nodeName,YG_NET_NODE);
    //distribute the transformName string
    addNetMessage("transform",&transformName,YG_NET_NODE);
    group = NULL;
    transform = NULL;
    }


void reference::reset(void)
    {
    //remove any previously referenced node
    if (group)
        pfnode()->removeChild(group);
    group = NULL;
    transform = NULL;
    //clear nodeName string
    resetNetMessage("node");
    //clear transformName string
    resetNetMessage("transform");
    ygNode::reset();
    }


void reference::message(const ygMessage& msg)
    {
    //set the name of the node to reference
    if (msg == "node")
        {
        //remove any previously referenced node
        if (group)
            pfnode()->removeChild(group);
        //if the nodeName string is not empty then
        if (nodeName.length() > 0)
            {
            //find the associated node in the database
            ygNode* node = ygNodeDB::find(nodeName);
            if (node)
                {
                if (node->className() == "ygWorld")
                    group = ygWorld::world()->debugNode();
                else
                    group = node->pfnode();
                }            
            else
                msg.error(name(),"(node not found)");
            //if the node was found then add the associated Performer node as a child
            if (group)
                pfnode()->addChild(group);
            }
        else
            group = NULL;
        }
    //set the name of the transform to reference
    else if (msg == "transform")
        {
        //if the nodeName string is not empty then
        if (transformName.length() > 0)
            {
            //find the associated node in the database
            ygNode* node;
            node = ygNodeDB::find(transformName);
            if (node)
                {
                if (node->isOfClass("ygTransform") || node->isOfClass("valuator"))
                    transform = node;
                else
                    msg.error(name(),"(node must be of class ygTransform or valuator)");
                }
            else
                msg.error(name(),"(node not found)");
            }
        else
            transform = NULL;
        }
    else
        ygNode::message(msg);
    }


void reference::app(void)
    {
    if (transform)
        {
        pfMatrix mat;
        ((pfDCS*)transform->pfnode())->getMat(mat);
        dcs->setMat(mat);
        }
    ygNode::app();
    }
