// Description: a space that generates events in the presence of a user head
//
//<b>notes:</b>
//<ul>
//<li> derived nodes inherit all messages from their base classes
//<li> see reset method for default settings
//<li> for volume messages refer to the <a href="ygSpace.html">ygSpace</a> node
//<li> this node requires the user have a <a href="ygHead.html">ygHead</a> node
//<li> the position mode determines if the reported position is local to the object or relative to world coordinates
//</ul>
//
// Category: Trigger
// Author: Alex Hill
// Revision: 05/10/02
//
#include <ygUser.h>
#include <ygWorld.h>
#include <ygUtil.h>
#include <ygNetMessages.h>
#include "userTrigger.h"

extern "C" ygNode* APIENTRY construct_userTrigger(const char* name,bool master) { return new userTrigger(name,master); }

userTrigger::userTrigger(const char* name,bool master) : ygSpace(name,master)
    {
    setClassName("userTrigger");
    debugFlag("inside", &debugInside);
    local = true;
    addNetMessage("local",&local,YG_NET_BOOL);
    addEvent("enter");
    addEvent("exit");
    addEvent("firstEnter");
    addEvent("empty");
    addEvent("inside");
    }

userTrigger::~userTrigger(void)
    {
    }

void userTrigger::reset(void)
    {
    //clear set of users previously inside
    prevInside.clear();
    //set reporting mode to local
    local = true;
    lastFrame = ygWorld::frameNumber();
    ygSpace::reset();
    }

void userTrigger::message(const ygMessage& msg)
    {
    //set position reporting mode
    if (msg == "local")
        {
        return;
        }
    else
        ygSpace::message(msg);
    }

void userTrigger::app(void)
    {
    set<ygUser*> inside;
    int i;
    if (lastFrame < (ygWorld::frameNumber()-1))
        prevInside.clear();
    lastFrame = ygWorld::frameNumber();
    //for each user in the world
    for (i=0; i < ygWorld::world()->numUsers(); i++)
        {
        ygUser * user = ygWorld::world()->user(i);
        //if user has a head then
        if (user->head())
            {
            //if head is within the space then
            pfVec3 pos = user->head()->origin(this);
            if (contains(pos))
                {
                if (debugInside)
                    cout << name() << ": " << user->name() << " is inside\n";
                //add to set of users inside
                inside.insert(inside.begin(),user);
                ygString args("user=");
                args += user->name();
                //if inside but not previously then generate enter event
                if (prevInside.find(user) == prevInside.end())
                    {
                    //a user has entered the space
                    eventOccurred("enter",args);
                    }
                if (!local)
                    pos = user->head()->origin();
                char value[32];
                args += " xpos=";
                sprintf(value,"%f",pos[0]);
                args += value;
                args += " ypos=";
                sprintf(value,"%f",pos[1]);
                args += value;
                args += " zpos=";
                sprintf(value,"%f",pos[2]);
                args += value;
                //a user is inside the space
                eventOccurred("inside",args);
                }
            }
        }
    set<ygUser*>::const_iterator iter;
    //for each user previously inside
    for (iter = prevInside.begin(); iter != prevInside.end();++iter)
        {
        //if not currently inside then generate exit event
        if (inside.find(*iter) == inside.end())
            {
            ygString args("user=");
            args += (*iter)->name();
            //a user has exited the space
            eventOccurred("exit",args);
            }
        }
    //if not previously empty and now empty generate empty event
    if ((prevInside.size() > 0) && (inside.size() == 0))
        {
        //all users have exited the space
        eventOccurred("empty");
        }
    //if previously empty and now not empty generate firstenter event
    if ((prevInside.size() == 0) && (inside.size() > 0))
        {
        ygString args("user=");
        args += (*(inside.begin()))->name();
        //a user has entered a previously empty space
        eventOccurred("firstenter",args);
        }
    prevInside = inside;
    ygSpace::app();
    }
