// Description: maintains a cache of Performer objects
//
//           11/01/01 Dave Pape
// Revision: 06/28/05 Alex Hill - added makeUnique function to dereference shared attributes
//
#include <Performer/pfdu.h>
#include <Performer/pf/pfGroup.h>
#include <Performer/pf/pfGeode.h>
#include <Performer/pr/pfMaterial.h>
#include <Performer/pr/pfGeoState.h>
#include <Performer/pr/pfGeoSet.h>
#include <map>
#include "ygString.h"
#include "ygPFObjectCache.h"

using namespace std;


struct _ygPFObjectCachePrivateData
    {
    map<ygString,pfNode*> objects;
    };


struct _ygPFObjectCachePrivateData* ygPFObjectCache::p_ = NULL;


pfNode* ygPFObjectCache::objectReference(const ygString& file)
    {
    pfNode *node;
    if (!p_)
        p_ = new struct _ygPFObjectCachePrivateData;
    node = p_->objects[file];
    if (!node)
        {
        node = pfdLoadFile(file.c_str());
        p_->objects[file] = node;
        }
    return node;
    }


pfNode* ygPFObjectCache::objectClone(const ygString& file)
    {
    pfNode * node = objectReference(file);
    if (node)
        return node->clone(0);
    else
        return NULL;
    }


void ygPFObjectCache::makeUnique(pfNode* node)
    {
    int i;
    //if node type is a pfGeode
    if (node->isOfType(pfGeode::getClassType()))
        {
        pfGeode *geode = (pfGeode *)node;
        //for each geoset
        for (i=0; i < geode->getNumGSets(); i++)
            {
            pfGeoSet* gSet = geode->getGSet(i);
            //duplicate geometry state to prevent material property sharing
            pfGeoState* gState = gSet->getGState();
            if (gState->getRef() > 2)
                {
                pfGeoState* newGState = new pfGeoState;
                newGState->copy(gState); 
                gSet->setGState(newGState);
                gState = newGState;
                }
            pfMaterial* frontMtl = (pfMaterial*)gState->getAttr(PFSTATE_FRONTMTL);
            if (frontMtl && frontMtl->getRef() > 2)
                {
                pfMaterial* newMtl = new pfMaterial;
                newMtl->copy(frontMtl);
                gState->setAttr(PFSTATE_FRONTMTL,newMtl);
                gState->setAttr(PFSTATE_BACKMTL,newMtl);
                frontMtl = newMtl;
                }
            pfMaterial* backMtl = (pfMaterial*)gState->getAttr(PFSTATE_BACKMTL);
            if (backMtl && (backMtl != frontMtl) && (backMtl->getRef() > 2))
                {
                pfMaterial* newMtl = new pfMaterial;
                newMtl->copy(backMtl);
                gState->setAttr(PFSTATE_BACKMTL,newMtl);
                }
            }
        }
    //else, if node type is a pfGroup
    else if (node->isOfType(pfGroup::getClassType()))
        {
        pfGroup *group = (pfGroup *)node;
        //call setGSetList recursively on all children
        for (i=0; i < group->getNumChildren(); i++)
            makeUnique(group->getChild(i));
        }
    }

