#ifndef _ygTransform_h_
#define _ygTransform_h_

#include <Performer/pr/pfLinMath.h>
#include <Performer/pf/pfDCS.h>
#include "ygNode.h"

#define COORD_NETKEY "x"
#define MATRIX_NETKEY "m"

using namespace std;

#include "ygDLL.h"

class YG_DLL_API ygTransform : public ygNode
    {
    public:
     ygTransform(const char*,bool master=true);
     virtual ~ygTransform(void);
     virtual void reset(void);
     virtual void message(const ygMessage&);
     virtual void app(void);
     virtual void setTransform(const pfMatrix&);
     void getMatrix(pfMatrix&) const;
     void getPosition(pfVec3&) const;
     void getOrientation(pfVec3&) const;
     void getSize(pfVec3&) const;
     void setMatrix(pfMatrix&);
     void setPosition(float x,float y,float z);
     void setOrientation(float x,float y,float z);
     void setSize(float s);
     void setSize(float sx,float sy,float sz);
     inline void setPosition(pfVec3 v) { setPosition(v[0],v[1],v[2]); }
     inline void setOrientation(pfVec3 v) {setOrientation(v[0],v[1],v[2]);}
     inline void setSize(pfVec3 v) { setSize(v[0],v[1],v[2]); }
    protected:
     virtual void createDebugNode(void);
    private:
     pfDCS* dcs;
     pfCoord netCoord;
     pfCoord lastCoord;
     pfCoord lastNetCoord;
     pfVec3 netSize;
     pfVec3 lastSize;
     pfVec3 lastNetSize;
     pfVec3 ori;
#if 0
    pfMatrix netMatrix;
#endif
     float updateInterval;
     bool netCoordAuto;
     bool netSizeAuto;
     int netCoordUnchanged;
     int netSizeUnchanged;
     float lastNetUpdate;
     float lastRadius;
     pfVec3* debugVerts;
     void setPosition(const ygMessage&);
     void setOrientation(const ygMessage&);
     void setDebugCoords(float);
    };

extern "C" YG_DLL_API ygNode* APIENTRY construct_ygTransform(const char*,bool);

#endif
