// Author: Dave Pape
//           11/01/01
//           06/28/05 Alex Hill - added depth option to ygPrintSceneGraph
// Revision: 02/01/06 Alex Hill - added ygIncrementName to avoid duplicate node names
//
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#ifndef _WIN32
#include <sys/time.h>
#else
#include "QUANTA\gettimeofday.hxx"  // from CAVERNSoft
#endif
#include "ygNode.h"
#include "ygUtil.h"

#include "ygDLL.h"

using namespace std;

bool APIENTRY ygFindFileInYgPath(const ygString& filename, ygString& returnPath)
    {
    static ygString searchPath;
    if (searchPath.length() < 1)
        {
        if (getenv("YG_PATH"))
            searchPath = getenv("YG_PATH");
        else
            searchPath = ".";
        }
    return ygFindFile(filename,searchPath,returnPath);
    }


bool APIENTRY ygFindFile(const ygString& filename, const ygString& searchPath,
                   ygString& returnPath)
    {
    ygString testName(filename);
    ygString *dir;
    int i=0;
    struct stat statbuf;
    if (stat(testName.c_str(),&statbuf) == 0)
        {
        returnPath = testName;
        return true;
        }
#ifdef WIN32
    while (dir = searchPath.nextToken(":;",&i))
        {
        testName = *dir;
        delete dir;
		testName += "\\";
        testName += filename;
		char fullPath[_MAX_PATH];

		_fullpath(fullPath, testName.c_str(), _MAX_PATH);
        if (stat(testName.c_str(),&statbuf) == 0)
            {
            returnPath = testName;
            return true;
            }
        }
    return false;
    }
#else
    while (dir = searchPath.nextToken(":",&i))
        {
        testName = *dir;
        delete dir;
        testName += "/";
        testName += filename;
        if (stat(testName.c_str(),&statbuf) == 0)
            {
            returnPath = testName;
            return true;
            }
        }
    return false;
    }
#endif

static bool ygTimeInitted=false;
static struct timeval ygTimeStart;
static float ygTimeMicrosecondF=1.0f/1000000.0f;
static double ygTimeMicrosecondD=1.0/1000000.0;

float APIENTRY ygGetTime(void)
    {
    struct timeval t;
    if (!ygTimeInitted)
        {
#ifndef WIN32        
        gettimeofday(&ygTimeStart,NULL);
#else
        quanta_gettimeofday(&ygTimeStart,NULL);
#endif        
        ygTimeInitted = true;
        }
#ifndef WIN32        
    gettimeofday(&t,NULL);
#else
    quanta_gettimeofday(&t,NULL);
#endif
    return (t.tv_sec - ygTimeStart.tv_sec) +
        (t.tv_usec - ygTimeStart.tv_usec) * ygTimeMicrosecondF;
    }

double APIENTRY ygGetTimeD(void)
    {
    struct timeval t;
    if (!ygTimeInitted)
        {
#ifndef WIN32        
        gettimeofday(&ygTimeStart,NULL);
#else
        quanta_gettimeofday(&ygTimeStart,NULL);
#endif        
        ygTimeInitted = true;
        }
#ifndef WIN32        
    gettimeofday(&t,NULL);
#else
    quanta_gettimeofday(&t,NULL);
#endif    
    return (t.tv_sec - ygTimeStart.tv_sec) +
        (t.tv_usec - ygTimeStart.tv_usec) * ygTimeMicrosecondD;
    }


static void APIENTRY printTree(ygNode* node,int depth,int max)
    {
    if (node && depth <= max)
        {
        int i;
        node->printMessages(cout,depth);
        for (i=0; i < node->numChildren(); i++)
            printTree(node->child(i),depth+1,max);
        }
    }


void APIENTRY ygPrintSceneGraph(ygNode* node,int max)
    {
    cout << "Scene Graph:\n";
    if (max < 0)
        {
        while(node->parent() && max < 0)
           {
           node = node->parent();
           max++;
           }
        max = 99;
        }
    printTree(node,0,max);
    }


void APIENTRY ygMakeValidNodeName(ygString& name)
    {
    char * s = name.c_str();
    int i;
    for (i=0; i < name.length(); i++)
        if (!isalnum(s[i]))
            s[i] = '_';
    }


void APIENTRY ygIncrementName(ygString& name)
    {
    char* s = name.c_str();
    int i;
    for (i=name.length()-1; i>0; i--)
        {
        if (!isdigit(s[i]))
            break;
        }
    int num = atoi(&s[i+1]);
    name.assign(name,0,i+1);
    num++;
    char index[32];
    sprintf(index,"%d",num);
    name += index;
    }

#ifdef __sgi

bool ygCreateThread(ygThreadFuncPtr func,void *arg)
    {
    pid_t p = sproc((void(*)(void*))func,PR_SADDR,arg);
    if (p <= 0)
        {
        perror("ygCreateThread: sproc");
        return false;
        }
    return true;
    }

#else

#include <QUANTA/QUANTA.hxx>

bool APIENTRY ygCreateThread(ygThreadFuncPtr func,void *arg)
    {
    QUANTAts_thread_c *thread = new QUANTAts_thread_c;
    int retval = thread->create(func,arg);
    return (retval == 0);
    }

#ifdef WIN32
void APIENTRY ygInit()
{
	QUANTAinit(); // to make ygGetTime, which uses quanta_gettimeofday internally, work correctly
}
#endif

#endif
