#ifndef _ygWorld_h_
#define _ygWorld_h_

#include "ygNode.h"

class ygUser;


#define YG_ISECT_ALLMASK      (1 << 0)
#define YG_ISECT_WALLMASK     (1 << 1)
#define YG_ISECT_FLOORMASK    (1 << 2)
#define YG_ISECT_GRABMASK     (1 << 3)
#define YG_ISECT_DEBUGMASK    (1 << 4)

#include "ygDLL.h"

class YG_DLL_API ygWorld : public ygNode
    {
    public:
     ygWorld(const char*,bool master=true);
     ~ygWorld(void);
     virtual void reset(void);
     virtual void message(const ygMessage&);
     virtual void app(void);
     void frame(void);
     void scheduleMessage(ygMessage&,bool queue=false);
     void createRoot(void);
     void setRoot(ygNode*);
     ygNode* root(void) const;
     class pfScene* pfscene(void) const;
     void setClip(float near,float far);
     void setSkyColor(const pfVec3&);
     void fogOff(void);
     void setFog(int fogtype,const pfVec3& color,float onset,float opaque);
     void addUser(ygUser*);
     void removeUser(ygUser*);
     int numUsers(void) const;
     ygUser* user(int) const;
     ygNode* viewer(void) const;
     void unlinkLocalScene(void);
     void deleteFromNode(ygNode*,bool include=false);
     void deleteLocalScene(void);
     void deleteTree(ygNode*);
     void saveToFile(const ygString&,ygNode*);
     void loadFromFile(const ygString&,ygNode*);
     void saveTree(std::ostream&,ygNode *,int);
     bool quit(void);
     static inline double frameTime(void) { return frameTime_; }
     static inline double frameDeltaTime(void) { return frameDeltaTime_; }
     static inline int frameNumber(void) { return frameNumber_; }
     static inline ygWorld* world(void) { return world_; }
    protected:
     virtual void updateDebug(void);
     virtual void createDebugNode(void);
    private:
     void sendMessage(ygMessage&);
     void sendClassMessage(const ygMessage&,const ygString&,ygNode *);
     void unlinkTree(ygNode*);
     class pfGeoSet* makeSphereGeoset(void);
     struct _ygWorldPrivateData * p_;
     static float frameTime_;
     static float frameDeltaTime_;
     static int frameNumber_;
     static ygWorld* world_;
    };

extern "C" YG_DLL_API ygNode* APIENTRY construct_ygWorld(const char* name,bool master);

#endif
